@@ -1,15 +1,16 @@
 
 /*  A Bison parser, made from bibparse.y
- by  GNU Bison version 1.25
-  */
+    by GNU Bison version 1.28  */
 
 #define YYBISON 1  /* Identify Bison output.  */
 
-#define	end_of_file	258
-#define	L_NAME	259
-#define	L_DIGIT	260
-#define	L_QUOTED	261
-#define	COMPOSE	262
+#define	end_of_file	257
+#define	L_NAME	258
+#define	L_DIGIT	259
+#define	L_COMMAND	260
+#define	L_BODY	261
+#define	L_SPACE	262
+#define	L_UBSPACE	263
 
 #line 1 "bibparse.y"
 
@@ -17,7 +18,7 @@
 /*
  This file is part of pybliographer
  
- Copyright (C) 1998 Frederic GOBRY
+ Copyright (C) 1998-1999 Frederic GOBRY
  Email : gobry@idiap.ch
  	   
  This program is free software; you can redistribute it and/or
@@ -34,73 +35,117 @@
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  
- $Id: bibparse.y,v 1.8 1998/10/28 13:29:11 gobry Exp $
+ $Id: bibparse.y,v 1.24 1999/07/30 12:34:45 gobry Exp $
 */
 
+#include "parsername.h"
 #include "bibtex.h"
 
-extern void bibtex_parser_initialize (bibtex_file *);
-extern void bibtex_parser_continue (bibtex_file *);
-extern void bibtex_parser_finish (bibtex_file *);
+extern void bibtex_parser_initialize (BibtexFile *);
+extern void bibtex_parser_continue (BibtexFile *);
+extern void bibtex_parser_finish (BibtexFile *);
+
+extern gboolean bibtex_parser_is_content;
 
 extern int yydebug;
 
-static bibtex_entry * entry = NULL;
-static int start_line;
-static bibtex_file * bfile;
+static BibtexEntry *	entry	= NULL;
+static int		start_line;
+static BibtexFile *	current_file;
+static GString *	error_string = NULL;
 
-static void nop (void) { return ;}
-
-void bibtex_next_line (void) { entry->length ++; }
-
-void bibtex_analyzer_initialize (bibtex_file * file)  {
-  bibtex_parser_initialize (file);
+static void 
+nop (void) { 
+    return ;
 }
 
-void bibtex_analyzer_finish (bibtex_file * file)  {
-  bibtex_parser_finish (file);
+void 
+bibtex_next_line (void) { 
+    entry->length ++; 
 }
 
-bibtex_entry * 
-parse_bibtex (bibtex_file * file) {
+void 
+bibtex_analyzer_initialize (BibtexFile * file)  {
+    bibtex_parser_initialize (file);
+}
+
+void 
+bibtex_analyzer_finish (BibtexFile * file)  {
+    g_return_if_fail (file != NULL);
+
+    bibtex_parser_finish (file);
+    
+    if (error_string)
+	g_string_free (error_string, TRUE);
+    
+    error_string = NULL;
+    current_file = NULL;
+}
+ 
+BibtexEntry * 
+parse_bibtex (BibtexFile * file) {
   int ret;
 
-  bfile = file;
+  g_return_val_if_fail (file != NULL, NULL);
+
+  if (! error_string) {
+      error_string = g_string_new (NULL);
+  }
+
+  current_file = file;
 
   yydebug = file->debug;
   start_line = file->line;
 
-  entry = g_new (bibtex_entry, 1);
-
-  entry->length = 0;
-  entry->name = entry->type = entry->body = NULL;
-  entry->table = g_hash_table_new (g_str_hash, g_str_equal);
+  entry = bibtex_entry_new ();
 
   bibtex_parser_continue (file);
+  bibtex_parser_is_content = FALSE;
+
   ret = yyparse ();
 
-  tmp_string_free ();
+  bibtex_tmp_string_free ();
 
   if (ret != 0) {
-    bibtex_free_entry (entry);
-    entry = NULL;
+      file->line += entry->length;
+      bibtex_entry_destroy (entry, TRUE);
+      entry = NULL;
   }
 
   return entry;
 }
 
- void yyerror (char * s) {
-   fprintf (stderr,"error: %d: %s\n", start_line + entry->length, s);
- }
- void yywarning (char * s) {
-   fprintf (stderr,"warning: %d: %s\n", start_line + entry->length, s);
- }
+void 
+yyerror (char * s) {
+    if (current_file) {
+	bibtex_error ("%s:%d: %s", current_file->filename,
+		      start_line + entry->length, s);
+    }
+    else {
+	bibtex_error ("%d: %s", 
+		      start_line + entry->length, s);
+
+    }
+}
+
+static void 
+yywarning (char * s) {
+    if (current_file) {
+	bibtex_warning ("%s:%d: %s", current_file->filename,
+			start_line + entry->length, s);
+    }
+    else {
+	bibtex_warning ("%d: %s", 
+			start_line + entry->length, s);
+
+    }
+}
 
 
-#line 87 "bibparse.y"
+#line 131 "bibparse.y"
 typedef union{
-  char * text;
-  bibtex_entry * entry;
+    gchar * text;
+    BibtexStruct * body;
 } YYSTYPE;
 #ifndef YYDEBUG
 #define YYDEBUG 1
@@ -116,26 +161,26 @@ typedef union{
 
 
 
-#define	YYFINAL		80
+#define	YYFINAL		51
 #define	YYFLAG		-32768
-#define	YYNTBASE	20
+#define	YYNTBASE	19
 
-#define YYTRANSLATE(x) ((unsigned)(x) <= 262 ? yytranslate[x] : 28)
+#define YYTRANSLATE(x) ((unsigned)(x) <= 263 ? yytranslate[x] : 31)
 
 static const char yytranslate[] = {     0,
-     2,     2,     2,     2,     2,     2,     2,     2,     2,    17,
      2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
      2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
-     2,    19,     2,    14,    12,     2,    10,     2,     2,     7,
-     8,     2,     2,     9,     2,     2,     2,     2,     2,     2,
      2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
-    11,     2,     2,    13,     2,     2,     2,     2,     2,     2,
+     2,     2,     2,    18,    17,     2,     2,     2,     2,    13,
+    14,     2,     2,    15,     2,     2,     2,     2,     2,     2,
+     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
+    16,     2,     2,    10,     2,     2,     2,     2,     2,     2,
      2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
      2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
      2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
      2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
      2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
-     2,     2,    15,     2,    16,     2,     2,     2,     2,     2,
+     2,     2,    11,     2,    12,     2,     2,     2,     2,     2,
      2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
      2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
      2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
@@ -148,44 +193,36 @@ static const char yytranslate[] = {     0,
      2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
      2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
      2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
-     2,     2,     2,     2,     2,     1,     2,     3,     4,     5,
-     6,    18
+     2,     2,     2,     2,     2,     1,     3,     4,     5,     6,
+     7,     8,     9
 };
 
 #if YYDEBUG != 0
 static const short yyprhs[] = {     0,
-     0,     6,    12,    14,    15,    19,    21,    25,    27,    31,
-    33,    35,    39,    41,    45,    47,    49,    53,    54,    57,
-    60,    63,    66,    69,    72,    75,    78,    81,    84,    87,
-    90,    91,    94,    97,   100,   103,   106,   109,   112,   115,
-   118,   121
+     0,     6,    12,    14,    20,    26,    30,    33,    35,    39,
+    41,    45,    47,    48,    53,    54,    59,    61,    63,    65,
+    67,    69,    73,    75,    77,    79,    80,    83,    86,    87
 };
 
-static const short yyrhs[] = {    13,
-     4,    15,    21,    16,     0,    13,     4,     7,    21,     8,
-     0,     3,     0,     0,    22,     9,    21,     0,    22,     0,
-     4,    11,    23,     0,     4,     0,    24,    12,    23,     0,
-    24,     0,     5,     0,    15,    26,    16,     0,     4,     0,
-    14,    27,    14,     0,     5,     0,     4,     0,    15,    26,
-    16,     0,     0,     6,    26,     0,     7,    26,     0,     8,
-    26,     0,     9,    26,     0,    14,    26,     0,    10,    26,
-     0,    11,    26,     0,    12,    26,     0,    13,    26,     0,
-    19,    26,     0,    17,    26,     0,    25,    26,     0,     0,
-     6,    27,     0,     7,    27,     0,     8,    27,     0,     9,
-    27,     0,    10,    27,     0,    11,    27,     0,    12,    27,
-     0,    13,    27,     0,    19,    27,     0,    17,    27,     0,
-    25,    27,     0
+static const short yyrhs[] = {    10,
+     4,    11,    20,    12,     0,    10,     4,    13,    20,    14,
+     0,     3,     0,    10,     4,    13,     1,    14,     0,    10,
+     4,    11,     1,    12,     0,    21,    15,    20,     0,    21,
+    15,     0,    21,     0,     4,    16,    22,     0,    22,     0,
+    27,    17,    22,     0,    27,     0,     0,    11,    24,    29,
+    12,     0,     0,    18,    26,    30,    18,     0,     5,     0,
+     4,     0,    23,     0,    25,     0,     6,     0,    11,    29,
+    12,     0,     8,     0,     9,     0,     7,     0,     0,    18,
+    29,     0,    28,    29,     0,     0,    28,    30,     0
 };
 
 #endif
 
 #if YYDEBUG != 0
 static const short yyrline[] = { 0,
-   124,   125,   126,   130,   131,   132,   136,   166,   175,   176,
-   179,   180,   181,   206,   209,   210,   211,   214,   215,   216,
-   217,   218,   219,   220,   221,   222,   223,   224,   225,   226,
-   229,   230,   231,   232,   233,   234,   235,   236,   237,   238,
-   239,   240
+   160,   169,   178,   185,   192,   206,   212,   218,   230,   278,
+   294,   300,   313,   314,   329,   330,   342,   349,   356,   362,
+   372,   379,   387,   393,   400,   411,   418,   427,   437,   444
 };
 #endif
 
@@ -193,89 +230,76 @@ static const short yyrline[] = { 0,
 #if YYDEBUG != 0 || defined (YYERROR_VERBOSE)
 
 static const char * const yytname[] = {   "$","error","$undefined.","end_of_file",
-"L_NAME","L_DIGIT","L_QUOTED","'('","')'","','","'%'","'='","'#'","'@'","'\"'",
-"'{'","'}'","'\\n'","COMPOSE","' '","entry","values","value","content","ext_text_part",
-"text_part","inner","inner2", NULL
+"L_NAME","L_DIGIT","L_COMMAND","L_BODY","L_SPACE","L_UBSPACE","'@'","'{'","'}'",
+"'('","')'","','","'='","'#'","'\\\"'","entry","values","value","content","content_brace",
+"@1","content_quote","@2","simple_content","text_part","text_brace","text_quote", NULL
 };
 #endif
 
 static const short yyr1[] = {     0,
-    20,    20,    20,    21,    21,    21,    22,    22,    23,    23,
-    24,    24,    24,    24,    25,    25,    25,    26,    26,    26,
-    26,    26,    26,    26,    26,    26,    26,    26,    26,    26,
-    27,    27,    27,    27,    27,    27,    27,    27,    27,    27,
-    27,    27
+    19,    19,    19,    19,    19,    20,    20,    20,    21,    21,
+    22,    22,    24,    23,    26,    25,    27,    27,    27,    27,
+    28,    28,    28,    28,    28,    29,    29,    29,    30,    30
 };
 
 static const short yyr2[] = {     0,
-     5,     5,     1,     0,     3,     1,     3,     1,     3,     1,
-     1,     3,     1,     3,     1,     1,     3,     0,     2,     2,
-     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
-     0,     2,     2,     2,     2,     2,     2,     2,     2,     2,
-     2,     2
+     5,     5,     1,     5,     5,     3,     2,     1,     3,     1,
+     3,     1,     0,     4,     0,     4,     1,     1,     1,     1,
+     1,     3,     1,     1,     1,     0,     2,     2,     0,     2
 };
 
 static const short yydefact[] = {     0,
-     3,     0,     0,     4,     4,     8,     0,     6,     0,     0,
-     2,     4,     1,    13,    11,    31,    18,     7,    10,     5,
-    16,    15,    31,    31,    31,    31,    31,    31,    31,    31,
-    18,    31,    31,    31,     0,    18,    18,    18,    18,    18,
-    18,    18,    18,    18,    18,    18,    18,     0,     0,    32,
-    33,    34,    35,    36,    37,    38,    39,     0,    41,    40,
-    42,    14,    19,    20,    21,    22,    24,    25,    26,    27,
-    23,    29,    28,    30,    12,     9,    17,     0,     0,     0
+     3,     0,     0,     0,     0,     0,    18,    17,    13,    15,
+     0,     8,    10,    19,    20,    12,     0,     0,     5,     0,
+    26,    29,     1,     7,     0,     4,     2,    18,     9,    21,
+    25,    23,    24,    26,    26,    26,     0,    29,     0,     6,
+    11,     0,    27,    28,    14,    30,    16,    22,     0,     0,
+     0
 };
 
-static const short yydefgoto[] = {    78,
-     7,     8,    18,    19,    47,    48,    35
+static const short yydefgoto[] = {    49,
+    11,    12,    13,    14,    21,    15,    22,    16,    36,    37,
+    39
 };
 
-static const short yypact[] = {     2,
--32768,    -3,    69,    -2,    -2,    -8,    -4,    11,   -10,    76,
--32768,    -2,-32768,-32768,-32768,    48,    32,-32768,     9,-32768,
--32768,-32768,    48,    48,    48,    48,    48,    48,    48,    48,
-    32,    48,    48,    48,     8,    32,    32,    32,    32,    32,
-    32,    32,    32,    32,    32,    32,    32,     7,    76,-32768,
--32768,-32768,-32768,-32768,-32768,-32768,-32768,    34,-32768,-32768,
--32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,
--32768,-32768,-32768,-32768,-32768,-32768,-32768,    62,    64,-32768
+static const short yypact[] = {    30,
+-32768,    -2,    24,     0,     2,    26,     6,-32768,-32768,-32768,
+    27,    21,-32768,-32768,-32768,    29,    34,    35,-32768,     5,
+    23,    36,-32768,    10,     5,-32768,-32768,-32768,-32768,-32768,
+-32768,-32768,-32768,    23,    23,    23,    38,    36,    33,-32768,
+-32768,    40,-32768,-32768,-32768,-32768,-32768,-32768,    53,    54,
+-32768
 };
 
 static const short yypgoto[] = {-32768,
-    77,-32768,    -1,-32768,   -16,   -12,    45
-};
-
-
-#define	YYLAST		91
-
-
-static const short yytable[] = {    34,
-     3,     6,    10,    11,     1,    13,    34,    34,    34,    34,
-    34,    34,    34,    34,     2,    34,    34,    34,    58,    12,
-    49,    62,    75,    63,    64,    65,    66,    67,    68,    69,
-    70,    71,    72,    73,    74,    21,    22,    36,    37,    38,
-    39,    40,    41,    42,    43,    44,    31,    76,    45,    77,
-    46,    21,    22,    23,    24,    25,    26,    27,    28,    29,
-    30,    79,    31,    80,    32,     0,    33,    50,    51,    52,
-    53,    54,    55,    56,    57,     4,    59,    60,    61,    14,
-    15,     9,     0,     5,     0,     0,     0,     0,    20,    16,
+    -5,-32768,    -8,-32768,-32768,-32768,-32768,-32768,   -14,    -9,
     17
 };
 
-static const short yycheck[] = {    16,
-     4,     4,    11,     8,     3,    16,    23,    24,    25,    26,
-    27,    28,    29,    30,    13,    32,    33,    34,    31,     9,
-    12,    14,    16,    36,    37,    38,    39,    40,    41,    42,
-    43,    44,    45,    46,    47,     4,     5,     6,     7,     8,
-     9,    10,    11,    12,    13,    14,    15,    49,    17,    16,
-    19,     4,     5,     6,     7,     8,     9,    10,    11,    12,
-    13,     0,    15,     0,    17,    -1,    19,    23,    24,    25,
-    26,    27,    28,    29,    30,     7,    32,    33,    34,     4,
-     5,     5,    -1,    15,    -1,    -1,    -1,    -1,    12,    14,
-    15
+
+#define	YYLAST		55
+
+
+static const short yytable[] = {    18,
+     6,     3,    17,     7,     8,     7,     8,    38,    28,     8,
+     9,    29,     9,     7,     8,     9,    41,    10,    40,    10,
+     9,    20,    10,    38,    42,    43,    44,    10,    30,    31,
+    32,    33,     1,    34,     4,    24,     5,    19,    23,     2,
+    35,    30,    31,    32,    33,    25,    34,    26,    27,    45,
+    47,    48,    50,    51,    46
+};
+
+static const short yycheck[] = {     5,
+     1,     4,     1,     4,     5,     4,     5,    22,     4,     5,
+    11,    20,    11,     4,     5,    11,    25,    18,    24,    18,
+    11,    16,    18,    38,    34,    35,    36,    18,     6,     7,
+     8,     9,     3,    11,    11,    15,    13,    12,    12,    10,
+    18,     6,     7,     8,     9,    17,    11,    14,    14,    12,
+    18,    12,     0,     0,    38
 };
 /* -*-C-*-  Note some compilers choke on comments on `#line' lines.  */
 #line 3 "/usr/local/share/bison.simple"
+/* This file comes from bison-1.28.  */
 
 /* Skeleton output parser for bison,
    Copyright (C) 1984, 1989, 1990 Free Software Foundation, Inc.
@@ -292,47 +316,67 @@ static const short yycheck[] = {    16,
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   Foundation, Inc., 59 Temple Place - Suite 330,
+   Boston, MA 02111-1307, USA.  */
 
 /* As a special exception, when this file is copied by Bison into a
    Bison output file, you may use that output file without restriction.
    This special exception was added by the Free Software Foundation
    in version 1.24 of Bison.  */
 
-#ifndef alloca
-#ifdef __GNUC__
-#define alloca __builtin_alloca
-#else /* not GNU C.  */
-#if (!defined (__STDC__) && defined (sparc)) || defined (__sparc__) || defined (__sparc) || defined (__sgi)
-#include <alloca.h>
-#else /* not sparc */
-#if defined (MSDOS) && !defined (__TURBOC__)
-#include <malloc.h>
-#else /* not MSDOS, or __TURBOC__ */
-#if defined(_AIX)
-#include <malloc.h>
- #pragma alloca
-#else /* not MSDOS, __TURBOC__, or _AIX */
-#ifdef __hpux
-#ifdef __cplusplus
-extern "C" {
-void *alloca (unsigned int);
-};
-#else /* not __cplusplus */
-void *alloca ();
-#endif /* not __cplusplus */
-#endif /* __hpux */
-#endif /* not _AIX */
-#endif /* not MSDOS, or __TURBOC__ */
-#endif /* not sparc.  */
-#endif /* not GNU C.  */
-#endif /* alloca not defined.  */
-
 /* This is the parser code that is written into each bison parser
   when the %semantic_parser declaration is not specified in the grammar.
   It was written by Richard Stallman by simplifying the hairy parser
   used when %semantic_parser is specified.  */
 
+#ifndef YYSTACK_USE_ALLOCA
+#ifdef alloca
+#define YYSTACK_USE_ALLOCA
+#else /* alloca not defined */
+#ifdef __GNUC__
+#define YYSTACK_USE_ALLOCA
+#define alloca __builtin_alloca
+#else /* not GNU C.  */
+#if (!defined (__STDC__) && defined (sparc)) || defined (__sparc__) || defined (__sparc) || defined (__sgi) || (defined (__sun) && defined (__i386))
+#define YYSTACK_USE_ALLOCA
+#include <alloca.h>
+#else /* not sparc */
+/* We think this test detects Watcom and Microsoft C.  */
+/* This used to test MSDOS, but that is a bad idea
+   since that symbol is in the user namespace.  */
+#if (defined (_MSDOS) || defined (_MSDOS_)) && !defined (__TURBOC__)
+#if 0 /* No need for malloc.h, which pollutes the namespace;
+	 instead, just don't use alloca.  */
+#include <malloc.h>
+#endif
+#else /* not MSDOS, or __TURBOC__ */
+#if defined(_AIX)
+/* I don't know what this was needed for, but it pollutes the namespace.
+   So I turned it off.   rms, 2 May 1997.  */
+/* #include <malloc.h>  */
+ #pragma alloca
+#define YYSTACK_USE_ALLOCA
+#else /* not MSDOS, or __TURBOC__, or _AIX */
+#if 0
+#ifdef __hpux /* haible@ilog.fr says this works for HPUX 9.05 and up,
+		 and on HPUX 10.  Eventually we can turn this on.  */
+#define YYSTACK_USE_ALLOCA
+#define alloca __builtin_alloca
+#endif /* __hpux */
+#endif
+#endif /* not _AIX */
+#endif /* not MSDOS, or __TURBOC__ */
+#endif /* not sparc */
+#endif /* not GNU C */
+#endif /* alloca not defined */
+#endif /* YYSTACK_USE_ALLOCA not defined */
+
+#ifdef YYSTACK_USE_ALLOCA
+#define YYSTACK_ALLOC alloca
+#else
+#define YYSTACK_ALLOC malloc
+#endif
+
 /* Note: there must be only one dollar sign in this file.
    It is replaced by the list of actions, each action
    as one case of the switch.  */
@@ -341,8 +385,8 @@ void *alloca ();
 #define yyclearin	(yychar = YYEMPTY)
 #define YYEMPTY		-2
 #define YYEOF		0
-#define YYACCEPT	return(0)
-#define YYABORT 	return(1)
+#define YYACCEPT	goto yyacceptlab
+#define YYABORT 	goto yyabortlab
 #define YYERROR		goto yyerrlab1
 /* Like YYERROR except do call yyerror.
    This remains here temporarily to ease the
@@ -423,12 +467,12 @@ int yydebug;			/*  nonzero means print parse trace	*/
 #ifndef YYMAXDEPTH
 #define YYMAXDEPTH 10000
 #endif
-
-/* Prevent warning if -Wstrict-prototypes.  */
-#ifdef __GNUC__
-int yyparse (void);
-#endif
 
+/* Define __yy_memcpy.  Note that the size argument
+   should be passed with type unsigned int, because that is what the non-GCC
+   definitions require.  With GCC, __builtin_memcpy takes an arg
+   of type size_t, but it can handle unsigned int.  */
+
 #if __GNUC__ > 1		/* GNU C and GNU C++ define this.  */
 #define __yy_memcpy(TO,FROM,COUNT)	__builtin_memcpy(TO,FROM,COUNT)
 #else				/* not GNU C or C++ */
@@ -440,7 +484,7 @@ static void
 __yy_memcpy (to, from, count)
      char *to;
      char *from;
-     int count;
+     unsigned int count;
 {
   register char *f = from;
   register char *t = to;
@@ -455,10 +499,10 @@ __yy_memcpy (to, from, count)
 /* This is the most reliable way to avoid incompatibilities
    in available built-in functions on various systems.  */
 static void
-__yy_memcpy (char *to, char *from, int count)
+__yy_memcpy (char *to, char *from, unsigned int count)
 {
-  register char *f = from;
   register char *t = to;
+  register char *f = from;
   register int i = count;
 
   while (i-- > 0)
@@ -468,7 +512,7 @@ __yy_memcpy (char *to, char *from, int count)
 #endif
 #endif
 
-#line 196 "/usr/local/share/bison.simple"
+#line 217 "/usr/local/share/bison.simple"
 
 /* The user can define YYPARSE_PARAM as the name of an argument to be passed
    into yyparse.  The argument should have type void *.
@@ -489,6 +533,15 @@ __yy_memcpy (char *to, char *from, int count)
 #define YYPARSE_PARAM_DECL
 #endif /* not YYPARSE_PARAM */
 
+/* Prevent warning if -Wstrict-prototypes.  */
+#ifdef __GNUC__
+#ifdef YYPARSE_PARAM
+int yyparse (void *);
+#else
+int yyparse (void);
+#endif
+#endif
+
 int
 yyparse(YYPARSE_PARAM_ARG)
      YYPARSE_PARAM_DECL
@@ -517,6 +570,7 @@ yyparse(YYPARSE_PARAM_ARG)
 #endif
 
   int yystacksize = YYINITDEPTH;
+  int yyfree_stacks = 0;
 
 #ifdef YYPURE
   int yychar;
@@ -601,18 +655,32 @@ yynewstate:
       if (yystacksize >= YYMAXDEPTH)
 	{
 	  yyerror("parser stack overflow");
+	  if (yyfree_stacks)
+	    {
+	      free (yyss);
+	      free (yyvs);
+#ifdef YYLSP_NEEDED
+	      free (yyls);
+#endif
+	    }
 	  return 2;
 	}
       yystacksize *= 2;
       if (yystacksize > YYMAXDEPTH)
 	yystacksize = YYMAXDEPTH;
-      yyss = (short *) alloca (yystacksize * sizeof (*yyssp));
-      __yy_memcpy ((char *)yyss, (char *)yyss1, size * sizeof (*yyssp));
-      yyvs = (YYSTYPE *) alloca (yystacksize * sizeof (*yyvsp));
-      __yy_memcpy ((char *)yyvs, (char *)yyvs1, size * sizeof (*yyvsp));
+#ifndef YYSTACK_USE_ALLOCA
+      yyfree_stacks = 1;
+#endif
+      yyss = (short *) YYSTACK_ALLOC (yystacksize * sizeof (*yyssp));
+      __yy_memcpy ((char *)yyss, (char *)yyss1,
+		   size * (unsigned int) sizeof (*yyssp));
+      yyvs = (YYSTYPE *) YYSTACK_ALLOC (yystacksize * sizeof (*yyvsp));
+      __yy_memcpy ((char *)yyvs, (char *)yyvs1,
+		   size * (unsigned int) sizeof (*yyvsp));
 #ifdef YYLSP_NEEDED
-      yyls = (YYLTYPE *) alloca (yystacksize * sizeof (*yylsp));
-      __yy_memcpy ((char *)yyls, (char *)yyls1, size * sizeof (*yylsp));
+      yyls = (YYLTYPE *) YYSTACK_ALLOC (yystacksize * sizeof (*yylsp));
+      __yy_memcpy ((char *)yyls, (char *)yyls1,
+		   size * (unsigned int) sizeof (*yylsp));
 #endif
 #endif /* no yyoverflow */
 
@@ -773,214 +841,260 @@ yyreduce:
   switch (yyn) {
 
 case 1:
-#line 124 "bibparse.y"
-{ entry->type = g_strdup (yyvsp[-3].text); g_strdown (entry->type); YYACCEPT; ;
-    break;}
-case 2:
-#line 125 "bibparse.y"
-{ entry->type = g_strdup (yyvsp[-3].text); g_strdown (entry->type); YYACCEPT; ;
-    break;}
-case 3:
-#line 126 "bibparse.y"
-{ bfile->eof = TRUE; YYABORT; ;
-    break;}
-case 4:
-#line 130 "bibparse.y"
-{ nop (); ;
-    break;}
-case 7:
-#line 136 "bibparse.y"
-{ 
-
-  char * name;
-  char * value;
-  int len;
-
-  len = strlen (yyvsp[-2].text);
-  while (yyvsp[-2].text[len - 1] == ' ') {
-    yyvsp[-2].text[len - 1] = '\0';
-    len --;
-  }
-
-  name = g_hash_table_lookup (entry->table, yyvsp[-2].text);
-  if (name) {
-    yywarning ("field redefinition");
-    g_free (name);
-
-    name = yyvsp[-2].text;
-  }
-  else {
-    name = g_strdup (yyvsp[-2].text);
-  }
-  value = g_strdup (yyvsp[0].text);
-  len = strlen (value);
-
-  if (value [len - 1] == '\n') value [len - 1] = '\0';
-  g_strdown (name);
-
-  g_hash_table_insert (entry->table, name, value); ;
-    break;}
-case 8:
-#line 166 "bibparse.y"
-{ 
-
-  if (entry->name) {
-    yyerror ("name redefinition");
-    YYABORT; 
-  }
-  entry->name = g_strdup (yyvsp[0].text); ;
-    break;}
-case 9:
-#line 175 "bibparse.y"
-{ yyval.text = merge_strings (yyvsp[-2].text, yyvsp[0].text); tmp_string (yyval.text); ;
-    break;}
-case 11:
-#line 179 "bibparse.y"
-{ yyval.text = yyvsp[0].text; ;
-    break;}
-case 12:
-#line 180 "bibparse.y"
-{ yyval.text = g_strconcat (yyvsp[-2].text, yyvsp[-1].text, yyvsp[0].text, NULL); tmp_string (yyval.text); ;
-    break;}
-case 13:
-#line 181 "bibparse.y"
+#line 162 "bibparse.y"
 {
+    entry->type = g_strdup (yyvsp[-3].text);
+    g_strdown (entry->type);
 
-	    gchar * val;
-	    gchar * key;
-	    
-	    if (yyvsp[0].text [0] == '{' || yyvsp[0].text [0] == '"') {
-	      yyval.text = yyvsp[0].text;
-	    }
-	    else {
-	      key = g_strdup (yyvsp[0].text);
-	      tmp_string (key);
-	      g_strdown (key);
-	      
-	      val = g_hash_table_lookup (bfile->table, key);
-	      
-	      if (val) {
-		yyval.text = g_strdup (val);
-		tmp_string (yyval.text);
-	      }
-	      else {
-		yywarning ("string undefined");
-		yyval.text = "\"\"";
-	      }
-	    }
+    YYACCEPT; 
 ;
     break;}
+case 2:
+#line 171 "bibparse.y"
+{ 
+    entry->type = g_strdup (yyvsp[-3].text);
+    g_strdown (entry->type);
+
+    YYACCEPT; 	
+;
+    break;}
+case 3:
+#line 180 "bibparse.y"
+{ 
+    current_file->eof = TRUE; 
+    YYABORT; 
+;
+    break;}
+case 4:
+#line 187 "bibparse.y"
+{
+    yyerror ("perhaps there is a missing coma");
+    YYABORT;
+;
+    break;}
+case 5:
+#line 194 "bibparse.y"
+{
+    yyerror ("perhaps there is a missing coma");
+    YYABORT;
+;
+    break;}
+case 6:
+#line 208 "bibparse.y"
+{
+    nop ();
+;
+    break;}
+case 7:
+#line 214 "bibparse.y"
+{
+    nop ();
+;
+    break;}
+case 8:
+#line 220 "bibparse.y"
+{
+    nop ();
+;
+    break;}
+case 9:
+#line 232 "bibparse.y"
+{ 
+    char * name;
+    BibtexField * field;
+    BibtexFieldType type = BIBTEX_OTHER;
+
+    g_strdown (yyvsp[-2].text);
+    field = g_hash_table_lookup (entry->table, yyvsp[-2].text);
+
+    /* Get a new instance of a field name */
+    if (field) {
+	g_string_sprintf (error_string, "field `%s' is already defined", yyvsp[-2].text); 
+	yywarning (error_string->str);
+
+	bibtex_field_destroy (field, TRUE);
+	name = yyvsp[-2].text;
+    }
+    else {
+	name = g_strdup (yyvsp[-2].text);
+    }
+
+    /* Search its type */
+    do {
+	if (strcmp (name, "author") == 0) {
+	    type = BIBTEX_AUTHOR;
+	    break;
+	}
+
+	if (strcmp (name, "title") == 0) {
+	    type = BIBTEX_TITLE;
+	    break;
+	}
+
+	if (strcmp (name, "year") == 0) {
+	    type = BIBTEX_DATE;
+	    break;
+	}
+    } 
+    while (0);
+
+    /* Convert into the right field */
+    field = bibtex_struct_as_field (bibtex_struct_flatten (yyvsp[0].body),
+				    type);
+
+    g_hash_table_insert (entry->table, name, field);
+;
+    break;}
+case 10:
+#line 280 "bibparse.y"
+{ 
+    if (entry->preamble) {
+	yyerror ("entry already contains a preamble");
+	YYABORT;
+    }
+
+    entry->preamble = yyvsp[0].body;
+;
+    break;}
+case 11:
+#line 296 "bibparse.y"
+{ 
+    yyval.body = bibtex_struct_append (yyvsp[-2].body, yyvsp[0].body);
+;
+    break;}
+case 12:
+#line 302 "bibparse.y"
+{
+    yyval.body = yyvsp[0].body;
+;
+    break;}
+case 13:
+#line 313 "bibparse.y"
+{ bibtex_parser_is_content = TRUE; ;
+    break;}
 case 14:
-#line 206 "bibparse.y"
-{ yyval.text = g_strconcat (yyvsp[-2].text, yyvsp[-1].text, yyvsp[0].text, NULL); tmp_string (yyval.text); ;
+#line 316 "bibparse.y"
+{ 
+    bibtex_parser_is_content = FALSE; 
+    yyval.body = bibtex_struct_new (BIBTEX_STRUCT_SUB);
+
+    yyval.body->value.sub->encloser = BIBTEX_ENCLOSER_BRACE;
+    yyval.body->value.sub->content  = yyvsp[-1].body;
+;
+    break;}
+case 15:
+#line 329 "bibparse.y"
+{ bibtex_parser_is_content = TRUE; ;
+    break;}
+case 16:
+#line 332 "bibparse.y"
+{ 
+    bibtex_parser_is_content = FALSE; 
+    yyval.body = bibtex_struct_new (BIBTEX_STRUCT_SUB);
+
+    yyval.body->value.sub->encloser = BIBTEX_ENCLOSER_QUOTE;
+    yyval.body->value.sub->content  = yyvsp[-1].body;
+;
     break;}
 case 17:
-#line 211 "bibparse.y"
-{ yyval.text = g_strconcat (yyvsp[-2].text, yyvsp[-1].text, yyvsp[0].text, NULL); tmp_string (yyval.text); ;
+#line 344 "bibparse.y"
+{ 
+    yyval.body = bibtex_struct_new (BIBTEX_STRUCT_TEXT);
+    yyval.body->value.text = g_strdup (yyvsp[0].text);
+;
     break;}
 case 18:
-#line 214 "bibparse.y"
-{ yyval.text = ""; ;
+#line 351 "bibparse.y"
+{
+    yyval.body = bibtex_struct_new (BIBTEX_STRUCT_REF);
+    yyval.body->value.ref = g_strdup (yyvsp[0].text);
+;
     break;}
 case 19:
-#line 215 "bibparse.y"
-{ yyval.text = g_strconcat (yyvsp[-1].text, yyvsp[0].text, NULL); tmp_string (yyval.text); ;
+#line 358 "bibparse.y"
+{ 
+    yyval.body = yyvsp[0].body;
+;
     break;}
 case 20:
-#line 216 "bibparse.y"
-{ yyval.text = g_strconcat (yyvsp[-1].text, yyvsp[0].text, NULL); tmp_string (yyval.text); ;
+#line 364 "bibparse.y"
+{ 
+    yyval.body = yyvsp[0].body;
+;
     break;}
 case 21:
-#line 217 "bibparse.y"
-{ yyval.text = g_strconcat (yyvsp[-1].text, yyvsp[0].text, NULL); tmp_string (yyval.text); ;
+#line 374 "bibparse.y"
+{ 
+    yyval.body = bibtex_struct_new (BIBTEX_STRUCT_COMMAND);
+    yyval.body->value.com = g_strdup (yyvsp[0].text + 1);
+;
     break;}
 case 22:
-#line 218 "bibparse.y"
-{ yyval.text = g_strconcat (yyvsp[-1].text, yyvsp[0].text, NULL); tmp_string (yyval.text); ;
+#line 381 "bibparse.y"
+{ 
+    yyval.body = bibtex_struct_new (BIBTEX_STRUCT_SUB);
+    yyval.body->value.sub->encloser = BIBTEX_ENCLOSER_BRACE;
+    yyval.body->value.sub->content  = yyvsp[-1].body;
+;
     break;}
 case 23:
-#line 219 "bibparse.y"
-{ yyval.text = g_strconcat (yyvsp[-1].text, yyvsp[0].text, NULL); tmp_string (yyval.text); ;
+#line 389 "bibparse.y"
+{
+    yyval.body = bibtex_struct_new (BIBTEX_STRUCT_SPACE);
+;
     break;}
 case 24:
-#line 220 "bibparse.y"
-{ yyval.text = g_strconcat (yyvsp[-1].text, yyvsp[0].text, NULL); tmp_string (yyval.text); ;
+#line 395 "bibparse.y"
+{
+    yyval.body = bibtex_struct_new (BIBTEX_STRUCT_SPACE);
+    yyval.body->value.unbreakable = TRUE;
+;
     break;}
 case 25:
-#line 221 "bibparse.y"
-{ yyval.text = g_strconcat (yyvsp[-1].text, yyvsp[0].text, NULL); tmp_string (yyval.text); ;
+#line 402 "bibparse.y"
+{
+    yyval.body = bibtex_struct_new (BIBTEX_STRUCT_TEXT);
+    yyval.body->value.text = g_strdup (yyvsp[0].text);
+;
     break;}
 case 26:
-#line 222 "bibparse.y"
-{ yyval.text = g_strconcat (yyvsp[-1].text, yyvsp[0].text, NULL); tmp_string (yyval.text); ;
+#line 413 "bibparse.y"
+{ 
+    yyval.body = bibtex_struct_new (BIBTEX_STRUCT_TEXT);
+    yyval.body->value.text = g_strdup ("");
+;
     break;}
 case 27:
-#line 223 "bibparse.y"
-{ yyval.text = g_strconcat (yyvsp[-1].text, yyvsp[0].text, NULL); tmp_string (yyval.text); ;
+#line 420 "bibparse.y"
+{ 
+    yyval.body = bibtex_struct_new (BIBTEX_STRUCT_TEXT);
+    yyval.body->value.text = g_strdup ("\"");
+
+    yyval.body = bibtex_struct_append (yyval.body, yyvsp[0].body);
+;
     break;}
 case 28:
-#line 224 "bibparse.y"
-{ yyval.text = g_strconcat (" ", yyvsp[0].text, NULL); tmp_string (yyval.text); ;
+#line 429 "bibparse.y"
+{ 
+    yyval.body = bibtex_struct_append (yyvsp[-1].body, yyvsp[0].body);
+;
     break;}
 case 29:
-#line 225 "bibparse.y"
-{ yyval.text = g_strconcat (" ", yyvsp[0].text, NULL); bibtex_next_line (); tmp_string (yyval.text);  ;
+#line 439 "bibparse.y"
+{ 
+    yyval.body = bibtex_struct_new (BIBTEX_STRUCT_TEXT);
+    yyval.body->value.text = g_strdup ("");
+;
     break;}
 case 30:
-#line 226 "bibparse.y"
-{ yyval.text = g_strconcat (yyvsp[-1].text, yyvsp[0].text, NULL); tmp_string (yyval.text); ;
-    break;}
-case 31:
-#line 229 "bibparse.y"
-{ yyval.text = ""; ;
-    break;}
-case 32:
-#line 230 "bibparse.y"
-{ yyval.text = g_strconcat (yyvsp[-1].text, yyvsp[0].text, NULL); tmp_string (yyval.text); ;
-    break;}
-case 33:
-#line 231 "bibparse.y"
-{ yyval.text = g_strconcat (yyvsp[-1].text, yyvsp[0].text, NULL); tmp_string (yyval.text); ;
-    break;}
-case 34:
-#line 232 "bibparse.y"
-{ yyval.text = g_strconcat (yyvsp[-1].text, yyvsp[0].text, NULL); tmp_string (yyval.text); ;
-    break;}
-case 35:
-#line 233 "bibparse.y"
-{ yyval.text = g_strconcat (yyvsp[-1].text, yyvsp[0].text, NULL); tmp_string (yyval.text); ;
-    break;}
-case 36:
-#line 234 "bibparse.y"
-{ yyval.text = g_strconcat (yyvsp[-1].text, yyvsp[0].text, NULL); tmp_string (yyval.text); ;
-    break;}
-case 37:
-#line 235 "bibparse.y"
-{ yyval.text = g_strconcat (yyvsp[-1].text, yyvsp[0].text, NULL); tmp_string (yyval.text); ;
-    break;}
-case 38:
-#line 236 "bibparse.y"
-{ yyval.text = g_strconcat (yyvsp[-1].text, yyvsp[0].text, NULL); tmp_string (yyval.text); ;
-    break;}
-case 39:
-#line 237 "bibparse.y"
-{ yyval.text = g_strconcat (yyvsp[-1].text, yyvsp[0].text, NULL); tmp_string (yyval.text); ;
-    break;}
-case 40:
-#line 238 "bibparse.y"
-{ yyval.text = g_strconcat (" ", yyvsp[0].text, NULL); tmp_string (yyval.text); ;
-    break;}
-case 41:
-#line 239 "bibparse.y"
-{ yyval.text = g_strconcat (" ", yyvsp[0].text, NULL); bibtex_next_line (); tmp_string (yyval.text);  ;
-    break;}
-case 42:
-#line 240 "bibparse.y"
-{ yyval.text = g_strconcat (yyvsp[-1].text, yyvsp[0].text, NULL); tmp_string (yyval.text); ;
+#line 446 "bibparse.y"
+{ 
+    yyval.body = bibtex_struct_append (yyvsp[-1].body, yyvsp[0].body);
+;
     break;}
 }
    /* the action file gets copied in in place of this dollarsign */
-#line 498 "/usr/local/share/bison.simple"
+#line 543 "/usr/local/share/bison.simple"
 
   yyvsp -= yylen;
   yyssp -= yylen;
@@ -1175,7 +1289,31 @@ yyerrhandle:
 
   yystate = yyn;
   goto yynewstate;
+
+ yyacceptlab:
+  /* YYACCEPT comes here.  */
+  if (yyfree_stacks)
+    {
+      free (yyss);
+      free (yyvs);
+#ifdef YYLSP_NEEDED
+      free (yyls);
+#endif
+    }
+  return 0;
+
+ yyabortlab:
+  /* YYABORT comes here.  */
+  if (yyfree_stacks)
+    {
+      free (yyss);
+      free (yyvs);
+#ifdef YYLSP_NEEDED
+      free (yyls);
+#endif
+    }
+  return 1;
 }
-#line 243 "bibparse.y"
+#line 453 "bibparse.y"
      
 	
